@@ -1,20 +1,32 @@
 """Miscellaneous functions and wrappers that didn't really have a place in any other submodules."""
 from __future__ import annotations
 
+__all__ = [
+    'DebugOutput', 'Thresholds', 'thresholding',
+    'fade_filter', 'merge_chroma',
+    'Planes', 'YUVPlanes', 'RGBPlanes',
+    'get_chroma_shift', 'get_bicubic_params',
+    'set_ffms2_log_level'
+]
+
 import math
 import warnings
 from abc import ABC
+from contextlib import AbstractContextManager
 from functools import partial, wraps
 from itertools import count
 from operator import ilshift, imatmul, ior
-from typing import (Any, Callable, ClassVar, Dict, Iterable, Iterator, List,
-                    MutableMapping, Optional, Tuple, Union, cast, overload)
+from types import TracebackType
+from typing import (
+    Any, Callable, ClassVar, Dict, Iterable, Iterator, List, Literal, MutableMapping, NamedTuple,
+    Optional, Sequence, Tuple, Type, TypeVar, Union, cast, overload
+)
 
 import vapoursynth as vs
-from lvsfunc.comparison import Stack
-from vsutil import get_w, insert_clip
+from lvsfunc.comparison import Direction, Stack
+from vsutil import depth, get_depth, get_w, insert_clip, join, plane
 
-from .types import F_OpInput, OpInput, Output
+from .types import F_OpInput, FormatError, OpInput, Output
 
 core = vs.core
 
@@ -44,7 +56,7 @@ class DebugOutputMMap(MutableMapping[int, vs.VideoNode], ABC):
     def __setitem__(self, index: int, clip: vs.VideoNode) -> None:
         self.outputs[index] = clip
 
-        self._min_idx, self._max_idx = min(self.outputs.keys()), max(self.outputs.keys())
+        self._update_minmax()
 
         if self._props:
             clip = clip.text.FrameProps(alignment=self._props, scale=self._scale)
@@ -55,15 +67,14 @@ class DebugOutputMMap(MutableMapping[int, vs.VideoNode], ABC):
 
     def __delitem__(self, index: int) -> None:
         del self.outputs[index]
-        self._min_idx, self._max_idx = min(self.outputs.keys()), max(self.outputs.keys())
+        self._update_minmax()
         vs.clear_output(index)
 
     def __len__(self) -> int:
         return len(self.outputs)
 
     def __iter__(self) -> Iterator[int]:
-        for i in self.outputs:
-            yield i
+        yield from self.outputs.keys()
 
     def __str__(self) -> str:
         string = ''
@@ -75,27 +86,31 @@ class DebugOutputMMap(MutableMapping[int, vs.VideoNode], ABC):
         return repr(self.outputs)
 
     def __del__(self) -> None:
+        """
+        Deleting an item will effectively freed the memory since we're invoking vs.clear_output.
+        Indexes are also updated.
+
+        However, we can't clear the outputs in the destructor of the DebugOutput instance.
+        Previewers won't be able to get the outputs because they run after the end of the script,
+        and the destructor is already used.
+
+        So ``del debug[0]`` will clear the output 0 but ``del debug`` won't.
+        If you want to clear outputs just do: ``debug.clear()`` and ``del debug``
+        """
         self.outputs.clear()
-        del self._min_idx
-        del self._max_idx
+        for name in set(self.__dict__):
+            delattr(self, name)
+
+    def _update_minmax(self) -> None:
+        try:
+            self._min_idx, self._max_idx = min(self.outputs.keys()), max(self.outputs.keys())
+        except ValueError:
+            del self._min_idx, self._max_idx
 
 
 class DebugOutput(DebugOutputMMap):
     """Utility class to ouput multiple clips"""
 
-    @overload
-    def __init__(self, *clips: Output) -> None:
-        """
-        Args:
-            clips (vs.VideoNode | List[vs.VideoNode] | Tuple[int, vs.VideoNode] | Tuple[int, List[vs.VideoNode]]):
-                `clips` can be a VideoNode, a list of planes,
-                a tuple of an index and VideoNode or a tuple of an index and a list of planes.
-                If a list of planes is passed, DebugOutput will try to stack the planes for previewing.
-                Only 444 and 420 format are allowed. Otherwise a warning will be raise and a garbage clip will be displayed.
-        """
-        ...
-
-    @overload
     def __init__(self, *clips: Output, props: int = 0, num: int = 0, scale: int = 1,
                  clear_outputs: bool = False, check_curr_env: bool = True, **named_clips: Output) -> None:
         """
@@ -130,10 +145,6 @@ class DebugOutput(DebugOutputMMap):
                 Check all clips set for output in the current environment.
                 Defaults to True.
         """
-        ...
-
-    def __init__(self, *clips: Output, props: int = 0, num: int = 0, scale: int = 1,
-                 clear_outputs: bool = False, check_curr_env: bool = True, **named_clips: Output) -> None:
         self._props = props
         self._num = num
         self._scale = scale
@@ -155,7 +166,7 @@ class DebugOutput(DebugOutputMMap):
 
         if clear_outputs:
             self.clear()
-            self.update(dict(rclips))
+            self.update(rclips)
         else:
             if check_curr_env:
                 self._check_curr_env(all_idx)
@@ -244,8 +255,7 @@ class DebugOutput(DebugOutputMMap):
 
     @staticmethod
     def _index_gen(start: int) -> Iterable[int]:
-        for i in count(start=start):
-            yield i
+        yield from count(start=start)
 
     @staticmethod
     def _stack_planes(planes: List[vs.VideoNode]) -> vs.VideoNode:
@@ -255,12 +265,11 @@ class DebugOutput(DebugOutputMMap):
                 format=vs.GRAY8, color=128
             ).text.Text('Problematic output: \noutput list out of range', 5, 2)
         else:
-            ws, hs = {c.width for c in planes}, {c.height for c in planes}
-            if len(ws) == len(hs) == 1:
+            if len({c.width for c in planes}) == len({c.height for c in planes}) == 1:
                 out = Stack(planes).clip
             else:
                 try:
-                    out = Stack([planes[0], Stack(planes[1:]).clip]).clip
+                    out = Stack([planes[0], Stack(planes[1:], direction=Direction.VERTICAL).clip]).clip
                 except ValueError:
                     warnings.warn('DebugOutput: unexpected subsampling')
                     out = core.std.BlankClip(
@@ -279,12 +288,96 @@ class DebugOutput(DebugOutputMMap):
         outputs: Dict[int, vs.VideoNode] = {}
         for idx, output in vs.get_outputs().items():
             if isinstance(output, vs.VideoOutputTuple):
-                outputs[idx] = output.clip
                 if output.alpha:
-                    warnings.warn('DebugOutput: VideoOutputTuple.alpha detected; this is not supported', Warning)
+                    outputs[idx] = output.clip.std.ClipToProp(output.alpha)
+                else:
+                    outputs[idx] = output.clip
         return outputs
 
 
+class Thresholds(NamedTuple):
+    """
+    [soft_bound_min, [hard_bound_min, hard_bound_max], soft_bound_max)
+    """
+    clip: vs.VideoNode
+    soft_bound_min: int | float | Sequence[int] | Sequence[float]
+    hard_bound_min: int | float | Sequence[int] | Sequence[float]
+    hard_bound_max: int | float | Sequence[int] | Sequence[float]
+    soft_bound_max: int | float | Sequence[int] | Sequence[float]
+    coef_min: int | float | Sequence[int] | Sequence[float] | None = None
+    coef_max: int | float | Sequence[int] | Sequence[float] | None = None
+
+
+def thresholding(*thrs: Thresholds, base: Optional[vs.VideoNode] = None, guidance: Optional[vs.VideoNode] = None) -> vs.VideoNode:
+    """
+    General function for applying specific filtering on specific thresholds
+    with gradation support before and after the hard thresholds
+
+    Args:
+        thrs (Thresholds):
+            Positional arguments of Thresholds.
+
+        base (vs.VideoNode, optional):
+            Base clip on which the first application will be made.
+            If not specified, a blank clip is made from the first ``thrs``.
+
+        guidance (VideoNode, optional):
+            Guidance clip on which the threshold references are taken.
+            If not specified, the guidance clip is made from the first ``thrs``.
+
+    Returns:
+        vs.VideoNode:
+            Thresholded clip.
+    """
+    if not base:
+        base = thrs[0].clip.std.BlankClip()
+    if not guidance:
+        guidance = thrs[0].clip
+
+    if not base.format or not guidance.format:
+        raise ValueError('thresholding: variable format not allowed')
+
+    for i, thr in enumerate(thrs):
+        if thr.clip.format != base.format:
+            raise ValueError(f'thresholding: threshold {i} has a different format than base clip')
+
+    def _normalise_thr(thr: int | float | Sequence[int] | Sequence[float], num_planes: int) -> List[int | float]:
+        thr = [thr] if isinstance(thr, (float, int)) else thr
+        return (list(thr) + [thr[-1]] * (num_planes - len(thr)))[:num_planes]
+
+    pclip = base
+
+    for thr in thrs:
+        soft_bound_min, hard_bound_min, hard_bound_max, soft_bound_max = (_normalise_thr(t, base.format.num_planes) for t in thr[1:5])
+        coef_min = _normalise_thr(thr.coef_min, base.format.num_planes) if thr.coef_min else None
+        coef_max = _normalise_thr(thr.coef_max, base.format.num_planes) if thr.coef_max else None
+
+        exprs: List[str] = []
+        for i in range(base.format.num_planes):
+            if_in_min = f'x {soft_bound_min[i]} >= x {hard_bound_min[i]} < and'
+            if_in_max = f'x {hard_bound_max[i]} >= x {soft_bound_max[i]} < and'
+            if_in_hard = f'x {hard_bound_min[i]} >= x {hard_bound_max[i]} < and'
+
+            str_min = f'x {soft_bound_min[i]} - {hard_bound_min[i]} {soft_bound_min[i]} - /'
+            if coef_min:
+                str_min += f' {coef_min[i]} pow'
+
+            str_max = f'x {hard_bound_max[i]} - {soft_bound_max[i]} {hard_bound_max[i]} - /'
+            if coef_max:
+                str_max += f' {coef_max} pow'
+
+            exprs.append(
+                if_in_min + f' z {str_min} * y 1 {str_min} - * + '
+                + if_in_max + f' y {str_max} * z 1 {str_max} - * + '
+                + if_in_hard + ' z y ? ? ?'
+            )
+
+        pclip = core.std.Expr([guidance, pclip, thr.clip], exprs)
+
+    return pclip
+
+
+
 def fade_filter(clip: vs.VideoNode, clip_a: vs.VideoNode, clip_b: vs.VideoNode,
                 start_f: int, end_f: int) -> vs.VideoNode:
     """Applies a filter by fading clip_a to clip_b.
@@ -327,6 +420,173 @@ def merge_chroma(luma: vs.VideoNode, ref: vs.VideoNode) -> vs.VideoNode:
     return core.std.ShufflePlanes([luma, ref], [0, 1, 2], vs.YUV)
 
 
+
+PlanesT = TypeVar('PlanesT', bound='Planes')
+
+
+class Planes(AbstractContextManager[vs.VideoNode], Sequence[vs.VideoNode]):
+    """General context manager for easier planes management"""
+
+    __slots__ = ('_clip', '_family', '_final_clip', '_planes')
+
+    def __init__(self, clip: vs.VideoNode, bits: Optional[int] = None, family: vs.ColorFamily = vs.YUV) -> None:
+        """
+        Args:
+            clip (vs.VideoNode):
+                Source clip
+
+            bits (Optional[int], optional):
+                Target bitdepth. Defaults to None.
+
+            family (vs.ColorFamily, optional):
+                Colour family. Defaults to vs.YUV.
+        """
+        self._clip = depth(clip, bits) if bits else clip
+        self._family = family
+        # Initialisation
+        self._final_clip: vs.VideoNode
+        self._planes: List[vs.VideoNode]
+        super().__init__()
+
+    def __enter__(self: PlanesT) -> PlanesT:
+        if isinstance(planes := self._clip.std.SplitPlanes(), Sequence):
+            self._planes = list(planes)
+        else:
+            raise FormatError(f'{self.__class__.__name__}: GRAY colour family isn\'t supported!')
+        return self
+
+    def __exit__(self, __exc_type: Type[BaseException] | None, __exc_value: BaseException | None,
+                 __traceback: TracebackType | None) -> bool | None:
+        self._final_clip = join(self._planes, self._family)
+        return super().__exit__(__exc_type, __exc_value, __traceback)
+
+    @overload
+    def __getitem__(self, i: int) -> vs.VideoNode:
+        ...
+
+    @overload
+    def __getitem__(self, i: slice) -> Sequence[vs.VideoNode]:
+        ...
+
+    def __getitem__(self, i: int | slice) -> vs.VideoNode | Sequence[vs.VideoNode]:
+        return self._planes[i]
+
+    def __setitem__(self, index: int, gray: vs.VideoNode) -> None:
+        try:
+            self._planes[index] = gray
+        except IndexError as i_err:
+            raise ValueError(f'{self.__class__.__name__}: plane number out of range') from i_err
+        if get_depth(gray) != (bits := get_depth(self._clip)):
+            # 32 bits float in YUV and doing on chroma planes
+            if bits == 32 and self._family == vs.YUV and index in {1, 2}:
+                gray = plane(depth(join([gray] * 3, self._family), bits), index)
+            else:
+                gray = depth(gray, bits)
+            self._planes[index] = depth(gray, bits)
+
+    def __delitem__(self, index: int) -> None:
+        self[index] = self[index].std.BlankClip()
+
+    def __len__(self) -> Literal[3]:
+        return 3
+
+    @property
+    def clip(self) -> vs.VideoNode:
+        """Get final merged clip"""
+        try:
+            out = self._final_clip
+        except AttributeError as attr_err:
+            raise ValueError(
+                f'{self.__class__.__name__}: you can only get "clip" outside of the context manager and once'
+            ) from attr_err
+        else:
+            del self._clip, self._family, self._final_clip, self._planes
+            return out
+
+
+class YUVPlanes(Planes):
+    def __init__(self, clip: vs.VideoNode, bits: Optional[int] = None) -> None:
+        super().__init__(clip, bits, vs.YUV)
+
+    @property
+    def Y(self) -> vs.VideoNode:
+        return self[0]
+
+    @Y.setter
+    def Y(self, _x: vs.VideoNode) -> None:
+        self[0] = _x
+
+    @Y.deleter
+    def Y(self) -> None:
+        del self[0]
+
+    @property
+    def U(self) -> vs.VideoNode:
+        return self[1]
+
+    @U.setter
+    def U(self, _x: vs.VideoNode) -> None:
+        self[1] = _x
+
+    @U.deleter
+    def U(self) -> None:
+        del self[1]
+
+    @property
+    def V(self) -> vs.VideoNode:
+        return self[2]
+
+    @V.setter
+    def V(self, _x: vs.VideoNode) -> None:
+        self[2] = _x
+
+    @V.deleter
+    def V(self) -> None:
+        del self[2]
+
+
+class RGBPlanes(Planes):
+    def __init__(self, clip: vs.VideoNode, bits: Optional[int] = None) -> None:
+        super().__init__(clip, bits, vs.RGB)
+
+    @property
+    def R(self) -> vs.VideoNode:
+        return self[0]
+
+    @R.setter
+    def R(self, _x: vs.VideoNode) -> None:
+        self[0] = _x
+
+    @R.deleter
+    def R(self) -> None:
+        del self[0]
+
+    @property
+    def G(self) -> vs.VideoNode:
+        return self[1]
+
+    @G.setter
+    def G(self, _x: vs.VideoNode) -> None:
+        self[1] = _x
+
+    @G.deleter
+    def G(self) -> None:
+        del self[1]
+
+    @property
+    def B(self) -> vs.VideoNode:
+        return self[2]
+
+    @B.setter
+    def B(self, _x: vs.VideoNode) -> None:
+        self[2] = _x
+
+    @B.deleter
+    def B(self) -> None:
+        del self[2]
+
+
+
 def get_chroma_shift(src_h: int, dst_h: int, aspect_ratio: float = 16 / 9) -> float:
     """Intended to calculate the right value for chroma shifting when doing subsampled scaling.
 
